/********************************************************
 *
 *
 * Dallas onewire support
 *
 *
 *
 * anhe, 10.11.2008
 *       03.03.2010 add DS1822 support
 *
 *
 ********************************************************/

#include "onewire.h"

#if USE_ONEWIRE
void long_delay_us(uint16_t ms)
{
    for (; ms>0; ms--) _delay_us(1);
}


bit w1_reset(void)
{
  bit ret;

  W1_DDR |= 1<<W1_PIN;		// bus write 
  W1_OUT &= ~(1<<W1_PIN);	// bus low 
  long_delay_us(480);		// wait 480 us
//  cli();
  W1_DDR &= ~(1<<W1_PIN);	// bus read
  long_delay_us(70);		// wait 70 us
  ret = W1_IN & (1<<W1_PIN);	// detect presence pulse (low) 60-240 us
//  sei();
  long_delay_us(480-70);	// wait after presence pulse (480-70) us
  if ((W1_IN & (1<<W1_PIN)) == 0)	// short circuit
    ret = 1;
  return ret;
}


unsigned char w1_bit_io( bit b )
{
//  cli();
  W1_DDR |= 1<<W1_PIN;		// bus write
  long_delay_us(1);
  if( b )
    W1_DDR &= ~(1<<W1_PIN);     // bus read
  long_delay_us(15-1);
  if( (W1_IN & (1<<W1_PIN)) == 0 )
    b = 0;
  long_delay_us(60-15);
  W1_DDR &= ~(1<<W1_PIN);	// bus read
//  sei();
  return b;
}


unsigned int w1_byte_wr( unsigned char b )
{
  unsigned char i = 8, j;
  do{
    j = w1_bit_io( b & 1 );
    b >>= 1;
    if( j )
      b |= 0x80;
  }while( --i );
  return b;
}


unsigned int w1_byte_rd( void )
{
  return w1_byte_wr( 0xFF );
}


//unsigned char w1_rom_search( unsigned char diff, unsigned char idata *id )
unsigned char w1_rom_search( unsigned char diff, unsigned char *id )
{
  unsigned char i, j, next_diff;
  bit b;

  if( w1_reset() )
    return PRESENCE_ERR;			// error, no device found
  w1_byte_wr( SEARCH_ROM );			// ROM search command
  next_diff = LAST_DEVICE;			// unchanged on last device
  i = 8 * 8;					// 8 bytes
  do{
    j = 8;					// 8 bits
    do{
      b = w1_bit_io( 1 );			// read bit
      if( w1_bit_io( 1 ) ){			// read complement bit
	if( b )					// 11
	  return DATA_ERR;			// data error
      }else{
	if( !b ){				// 00 = 2 devices
	  if( diff > i ||
	    ((*id & 1) && diff != i) ){
	    b = 1;				// now 1
	    next_diff = i;			// next pass 0
	  }
	}
      }
      w1_bit_io( b );     			// write bit
      *id >>= 1;
      if( b )					// store bit
	*id |= 0x80;
      i--;
    }while( --j );
    id++;					// next byte
  }while( i );
  return next_diff;				// to continue search
}


//void w1_command( unsigned char command, unsigned char idata *id )
void w1_command( unsigned char command, unsigned char *id )
{
  unsigned char i;
  w1_reset();
  if( id ){
    w1_byte_wr( MATCH_ROM );			// to a single device
    i = 8;
    do{
      w1_byte_wr( *id );
      id++;
    }while( --i );
  }else{
    w1_byte_wr( SKIP_ROM );			// to all devices
  }
  w1_byte_wr( command );
}


void start_meas( void ){
  if( W1_IN & 1<< W1_PIN ){
    w1_command( CONVERT_T, NULL );
    W1_OUT |= 1<< W1_PIN;
    W1_DDR |= 1<< W1_PIN;			// parasite power on
  }else{
    strcpy(ow.bus_info, "Short Circuit !");
  }
}


void read_meas( void )
{
  unsigned char id[8], diff;
  int i = 0;
  uint16_t temp, cpc, cr, crc, subzero, conf;

  for( diff = SEARCH_FIRST; diff != LAST_DEVICE; ){
    diff = w1_rom_search( diff, id );

    if( diff == PRESENCE_ERR ){
        strcpy(ow.bus_info, "No Sensor found");
      break;
    }
    if( diff == DATA_ERR ){
        strcpy(ow.bus_info, "Bus Error");
      break;
    }
    // zero error messages
    strcpy(ow.bus_info,"");
#if ONEWIRE_DS1820
    if( id[0] == 0x28 || id[0] == 0x10 ){	// DS1820 - temperature sensor
      sprintf(ow.dev[i].id, "%02X%02X%02X%02X%02X%02X%02X%02X", id[0],id[1],id[2],id[3],id[4],id[5],id[6],id[7]);
      // start reading
      w1_byte_wr( READ );			// read command
      temp = w1_byte_rd();			// Temp LSB
      temp |= (w1_byte_rd())<<8;		// Temp MSB

      if ( temp & 0x8000 ) {			// negativ temp.
        temp ^= 0xffff;			// invert temp
        temp++;
        subzero = 1;				// mark negativ temp
      } else {
	subzero = 0;
      }
      w1_byte_rd();			// User Byte 1
      w1_byte_rd();			// User Byte 2
      w1_byte_rd();			// Reserve
      w1_byte_rd();			// Reserve
      cr = (uint16_t)w1_byte_rd();	// Count Remain
      cpc = (uint16_t)w1_byte_rd();	// Count Per °C
      crc = (uint16_t)w1_byte_rd();	// CRC
      // calculating temperature with higher precision
      //temp = temp>>1; 	// trenne 0.5°C bit ab
      //temp = temp*100-25+(((cpc-cr)*100)/cpc);
      // debug
      //usart_write("DS18S20: %i %i -- %i %i %i\r\n", temp_low, temp_high, cr, cpc, crc);
      //usart_write("         %c%i.%i\r\n", (subzero)?'-':'+', temp/100, temp%100);
      //sprintf(ow.dev[i].value, "%c%i.%i", (subzero)?'-':'+', temp/100, temp%100); // display Temperatur
      // calculating temperature
      // temp & 0x0001 			// Nachkommastellen * 0.5 °C
      // temp>>1			// trenne Nachkommastellen ab
      sprintf(ow.dev[i].value, "%c%i.%i", (subzero)?'-':'+', temp>>1, (temp & 0x0001)*5); // display Temperatur
    }
#endif
#if ONEWIRE_DS1822
    if( id[0] == 0x22 ) {      // DS1822 - temperature sensor with 12bit at poweron
      sprintf(ow.dev[i].id, "%02X%02X%02X%02X%02X%02X%02X%02X", id[0],id[1],id[2],id[3],id[4],id[5],id[6],id[7]);
      // start reading
      w1_byte_wr( READ );                       // read command
      temp = w1_byte_rd();			// Temp LSB
      temp |= (w1_byte_rd())<<8;		// Temp MSB

      if ( temp & 0xf000 ) {			// negativ temp.
        temp ^= 0xffff;				// invert temp
        subzero = 1;				// mark negativ temp
      } else {
	subzero = 0;
      }
      w1_byte_rd();			// TH Register or User Byte 1
      w1_byte_rd();			// TL Register or User Byte 2
      conf = w1_byte_rd();		// Configuration Register
      w1_byte_rd();			// Reserve (0xFF)
      w1_byte_rd();			// Reserve
      w1_byte_rd();			// Reserve (0x10)
      crc = (uint16_t)w1_byte_rd();	// CRC
      // calculating temperature
      // temp & 0x000f 			// Nachkommastellen * 1/16 °C
      // temp>>4			// trenne Nachkommastellen ab
      sprintf(ow.dev[i].value, "%c%i.%i", (subzero)?'-':'+', temp>>4, ((temp & 0x000f)*100)/16); // display Temperatur
    }
#endif
    // increment ow-array[index]
    i++;
  } //for

  // zero rest ow_array
  while ( i< MAX_OW_DEVICES )
  {
    ow.dev[i].id[0] = '\0';
    ow.dev[i].value[0] = '\0';
    i++;
  }
}

#endif
